#include <stdio.h>

#include "gtall.h"

/************************************************************************\
*
* Buttons example part.
*
* $Copyright: (C) 1997 by Gran Gustafsson $
*
\************************************************************************/

#include "gtlogo2.bm"

static GtAppShell *shell = NULL;
static GtLabel *msg1,*msg2,*msg3;
static GtNumber *repnum = NULL;
static int repnr = 0;
static char buf1[256],buf2[256],buf3[256];
static char *strs[]=
{
	"Red",
	"Green",
	"Blue",
	"Yellow",
	"Orange",
	"Black",
	"White",
	NULL
};

static char *text[]=
{
	"\title",
	"Buttons Demo",
	"This is a demo that show some of the features of the",
	"buttons avaliable in the Gt library. In Gt all buttons",
	"are simply layout managers (inherits from GtGroup)",
	"and that allows for any contents of a button (even",
	"another button).",
	"\title",
	"Simple Buttons",
	"The simple buttons of this demo is from left to right:",
	"\nl",
	"1) A button with a GtLabel inside to show a string in",
	"the button. This button is set to allow presses of any",
	"mouse button. The three callback; arm, activate and",
	"disarm display a message in at the bottom of the window",
	"when they are invoked.",
	"\nl",
	"2) A button with two pixmap labels on each side of a",
	"text label.",
	"\nl",
	"3) A button with a GtLabel to display a string and a",
	"GtNumber widget to display a number. The button is set",
	"to call it's activate callback repeatedly (every 100",
	"milliseconds) and increment the number on each call.",
	"\title",
	"Toggle Buttons",
	"The two toggle buttons of this demo is from the top:",
	"\nl",
	"1) A standard toggle button with a GtSymbol as indicator",
	"of it's state (on = filled down, off = unfilled out).",
	"\nl",
	"2) A toggle button that use the border to indicate it's",
	"state (on = border in, off = border out).",
	"\title",
	"Radio Buttons",
	"The four radio buttons shows how all toggle buttons",
	"that is placed inside a GtRadioGroup turns into radio",
	"buttons, even if they are in separate groups nested",
	"inside the radiogroup. The two radio buttons on the",
	"second row uses their border to indicate the state.",
	"\title",
	"Cascade Buttons",
	"The cascade buttons in this demo shows how menus can",
	"be popped up from a cascade button in one of four",
	"directions. Cascade buttons can also be used in a",
	"menu to pop up a sub menu.",
	"\title",
	"Cycle Button",
	"The cycle buttons is a special instance of a cascade",
	"button with a automatically created menu with a set",
	"of menu items. The currently selected menu item is shown",
	"as the label of the cycle button. A new menu item may",
	"be selected by pulling down the menu with the right",
	"mouse button. The menu items can also be cycled through",
	"by pressing the left button in the cycle button.",
	NULL
};

/************************************************************************\
*
* Close buttons shell but do not destroy it.
*
\************************************************************************/
static int CloseCB(GtBase *obj,GtCDPtr cd,GtUDPtr ud)
{
	GtAppShell *s;
	
	obj = obj;
	cd = cd;
	s = (GtAppShell *)ud;
	s->Map(FALSE);
	return(GTCB_CONT);
}

/************************************************************************\
*
* Armed button.
*
\************************************************************************/
static int ArmCB(GtBase *obj,GtCDPtr cd,GtUDPtr ud)
{
	GtButtonCD *bcd;

	obj = obj;
	bcd = (GtButtonCD *)cd;
	ud = ud;
	sprintf(buf1,"Armed - button: %ld, state: %08lX, multiclick: %d",
		bcd->button,bcd->state,bcd->multi);
	msg1->SetV(GT_String,buf1,GT_DONE);
	msg2->SetV(GT_String,"",GT_DONE);
	msg3->SetV(GT_String,"",GT_DONE);
	return(GTCB_CONT);
}

/************************************************************************\
*
* Activate button.
*
\************************************************************************/
static int ActivateCB(GtBase *obj,GtCDPtr cd,GtUDPtr ud)
{
	GtButtonCD *bcd;

	obj = obj;
	bcd = (GtButtonCD *)cd;
	ud = ud;
	sprintf(buf2,"Activate - button: %ld, state: %08lX, multiclick: %d",
		bcd->button,bcd->state,bcd->multi);
	msg2->SetV(GT_String,buf2,GT_DONE);
	return(GTCB_CONT);
}

/************************************************************************\
*
* Disarmed button.
*
\************************************************************************/
static int DisarmCB(GtBase *obj,GtCDPtr cd,GtUDPtr ud)
{
	GtButtonCD *bcd;

	obj = obj;
	bcd = (GtButtonCD *)cd;
	ud = ud;
	sprintf(buf3,"Disarmed - button: %ld, state: %08lX, multiclick: %d",
		bcd->button,bcd->state,bcd->multi);
	msg3->SetV(GT_String,buf3,GT_DONE);
	return(GTCB_CONT);
}

/************************************************************************\
*
* Repeate button activate.
*
\************************************************************************/
static int RepeatCB(GtBase *obj,GtCDPtr cd,GtUDPtr ud)
{
	obj = obj;
	cd = cd;
	ud = ud;
	repnr = (repnr + 1) % 1000;
	repnum->SetV(GT_Number,repnr,GT_DONE);
	return(GTCB_CONT);
}

/************************************************************************\
*
* Setup buttons shell.
*
\************************************************************************/
void CreateButtons(GtAppShell *parent)
{
	GtGroup *g1,*g2,*g3,*g4;
	GtRadioGroup *rg;
	GtFrame *f;
	GtButton *b;
	GtToggle *t;
	GtCascade *c;
	GtMenu *menu;
	GtPixmap *pix;
	
	if(shell)
	{
		shell->Map(TRUE);
		return;
	}
	
	shell = GtMkAppShell("buttons",parent,NULL);
	shell->AddCB(GT_Close,CloseCB,shell);
	
	g1 = GtMkGroup("vert",shell,
		GT_Orient,GTO_VERT,
		GT_DONE);
	
	f = GtMkFrame("simplefrm",g1,NULL);
	GtMkLabel("simple",f,GT_FrameTitle,TRUE,GT_DONE);
	
	g2 = GtMkGroup("simplegrp",f,
		GT_Orient,GTO_HORIZ,
		GT_DONE);
	
	b = GtMkButton("blabel",g2,
		GT_ButtonMask,Button1Mask|Button2Mask|Button3Mask,
		GT_DONE);
	b->AddCB(GT_Arm,ArmCB,NULL);
	b->AddCB(GT_Activate,ActivateCB,NULL);
	b->AddCB(GT_Disarm,DisarmCB,NULL);
	GtMkLabel("label",b,NULL);
	
	b = GtMkButton("bmulti",g2,NULL);
	pix = GtMkPixmap(b->GetDisp(),
		GT_BitmapData,gtlogo2_bits,
		GT_XSize,gtlogo2_width,
		GT_YSize,gtlogo2_height,
		GT_DONE);
	GtMkLabel("logo",b,GT_Pixmap,pix,GT_DONE);
	GtMkLabel("multi",b,NULL);
	GtMkLabel("logo",b,GT_Pixmap,pix,GT_DONE);
	
	b = GtMkButton("brepeat",g2,
		GT_Delay,100,
		GT_Repeat,TRUE,
		GT_DONE);
	b->AddCB(GT_Activate,RepeatCB,NULL);
	GtMkLabel("repeat",b,NULL);
	repnum = GtMkNumber("repnum",b,
		GT_Number,repnr,
		GT_Format,"%03d",
		GT_DONE);

	g2 = GtMkGroup("trhoriz",g1,
		GT_Orient,GTO_HORIZ,
		GT_DONE);
	
	f = GtMkFrame("togglefrm",g2,NULL);
	GtMkLabel("toggle",f,GT_FrameTitle,TRUE,GT_DONE);
	
	g3 = GtMkGroup("trvert",f,
		GT_Orient,GTO_VERT,
		GT_DONE);
	
	t = GtMkToggle("t1",g3,NULL);
	GtMkLabel("toggle1",t,NULL);

	t = GtMkToggle("t2",g3,GT_ToggleBorder,TRUE,GT_DONE);
	GtMkLabel("toggle2",t,NULL);

	f = GtMkFrame("radiofrm",g2,NULL);
	GtMkLabel("radio",f,GT_FrameTitle,TRUE,GT_DONE);

	rg = GtMkRadioGroup("trradio",f,
		GT_Orient,GTO_HORIZ,
		GT_DONE);
	
	g3 = GtMkGroup("trvert",rg,
		GT_Orient,GTO_VERT,
		GT_DONE);
	
	t = GtMkToggle("r1",g3,NULL);
	GtMkLabel("radio1",t,NULL);

	t = GtMkToggle("r2",g3,GT_ToggleBorder,TRUE,GT_DONE);
	GtMkLabel("radio2",t,NULL);

	g3 = GtMkGroup("trvert",rg,
		GT_Orient,GTO_VERT,
		GT_DONE);
	
	t = GtMkToggle("r3",g3,NULL);
	GtMkLabel("radio3",t,NULL);

	t = GtMkToggle("r4",g3,GT_ToggleBorder,TRUE,GT_DONE);
	GtMkLabel("radio4",t,NULL);

	f = GtMkFrame("cascfrm",g1,NULL);
	GtMkLabel("cascade",f,GT_FrameTitle,TRUE,GT_DONE);

	g2 = GtMkGroup("cascgrp",f,
		GT_Orient,GTO_HORIZ,
		GT_DONE);
	
	menu = GtMkMenu("cascmenu",shell,NULL);
	g3 = GtMkGroup("cmgrp",menu,
		GT_Orient,GTO_VERT,
		GT_FrameType,GTF_OUT,
		GT_DONE);
	
	c = GtMkCascade("c1",g2,GT_Menu,menu,GT_Direction,GTD_LEFT,GT_DONE);
	GtMkLabel("casc1",c,NULL);

	c = GtMkCascade("c2",g2,GT_Menu,menu,GT_Direction,GTD_UP,GT_DONE);
	GtMkLabel("casc2",c,NULL);

	c = GtMkCascade("c3",g2,GT_Menu,menu,GT_Direction,GTD_DOWN,GT_DONE);
	GtMkLabel("casc3",c,NULL);

	c = GtMkCascade("c4",g2,GT_Menu,menu,GT_Direction,GTD_RIGHT,GT_DONE);
	GtMkLabel("casc4",c,NULL);

	b = GtMkButton("bdummy",g3,NULL);
	GtMkLabel("dummy",b,NULL);

	menu = GtMkMenu("cascmenu",menu,NULL);
	g4 = GtMkGroup("cmgrp",menu,
		GT_Orient,GTO_VERT,
		GT_FrameType,GTF_OUT,
		GT_DONE);

	b = GtMkButton("bdummy",g4,NULL);
	GtMkLabel("dummy",b,NULL);
	b = GtMkButton("bdummy",g4,NULL);
	GtMkLabel("dummy",b,NULL);
	b = GtMkButton("bdummy",g4,NULL);
	GtMkLabel("dummy",b,NULL);

	c = GtMkCascade("csub",g3,GT_Menu,menu,GT_DONE);
	GtMkLabel("submenu",c,NULL);

	b = GtMkButton("bdummy",g3,NULL);
	GtMkLabel("dummy",b,NULL);
	
	f = GtMkFrame("cyclefrm",g1,NULL);
	GtMkLabel("cycle",f,GT_FrameTitle,TRUE,GT_DONE);
	
	GtMkCycle("bcycle",f,GT_LabelNames,strs,GT_DONE);
	
	g2 = GtMkGroup("msgfrm",g1,
		GT_Weight,0,
		GT_Orient,GTO_VERT,
		GT_DONE);
	msg1 = GtMkLabel("msg1",g2,
		GT_String,"",
		GT_DONE);
	msg2 = GtMkLabel("msg2",g2,
		GT_String,"",
		GT_DONE);
	msg3 = GtMkLabel("msg3",g2,
		GT_String,"",
		GT_DONE);
	
	GtMkSeparator("sep",g1,NULL);
	
	b = GtMkButton("bok",g1,NULL);
	b->AddCB(GT_Activate,CloseCB,shell);
	GtMkLabel("ok",b,NULL);
	
	shell->Build();
}

/************************************************************************\
*
* Return the text for the buttons part.
*
\************************************************************************/
char **GetButtonsText(void)
{
	return(text);
}
